"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var lodash_1 = require("lodash");
var url_1 = require("url");
var call_1 = require("./call");
var channel_1 = require("./channel");
var constants_1 = require("./constants");
var metadata_1 = require("./metadata");
var Client = /** @class */ (function () {
    function Client(address, credentials, options) {
        if (options === void 0) { options = {}; }
        if (options['grpc.primary_user_agent']) {
            options['grpc.primary_user_agent'] += ' ';
        }
        else {
            options['grpc.primary_user_agent'] = '';
        }
        // TODO(murgatroid99): Figure out how to get version number
        // options['grpc.primary_user_agent'] += 'grpc-node/' + version;
        this.channel = new channel_1.Http2Channel(new url_1.URL(address), credentials, options);
    }
    Client.prototype.close = function () {
        this.channel.close();
    };
    Client.prototype.waitForReady = function (deadline, callback) {
        var cb = lodash_1.once(callback);
        var callbackCalled = false;
        this.channel.connect(function () {
            cb(null);
        });
        if (deadline !== Infinity) {
            var timeout = void 0;
            var now = (new Date).getTime();
            if (deadline instanceof Date) {
                timeout = deadline.getTime() - now;
            }
            else {
                timeout = deadline - now;
            }
            if (timeout < 0) {
                timeout = 0;
            }
            setTimeout(function () {
                cb(new Error('Failed to connect before the deadline'));
            }, timeout);
        }
    };
    Client.prototype.handleUnaryResponse = function (call, deserialize, callback) {
        var responseMessage = null;
        call.on('data', function (data) {
            if (responseMessage != null) {
                call.cancelWithStatus(constants_1.Status.INTERNAL, 'Too many responses received');
            }
            try {
                responseMessage = deserialize(data);
            }
            catch (e) {
                call.cancelWithStatus(constants_1.Status.INTERNAL, 'Failed to parse server response');
            }
        });
        call.on('end', function () {
            if (responseMessage == null) {
                call.cancelWithStatus(constants_1.Status.INTERNAL, 'Not enough responses received');
            }
        });
        call.on('status', function (status) {
            /* We assume that call emits status after it emits end, and that it
             * accounts for any cancelWithStatus calls up until it emits status.
             * Therefore, considering the above event handlers, status.code should be
             * OK if and only if we have a non-null responseMessage */
            if (status.code === constants_1.Status.OK) {
                callback(null, responseMessage);
            }
            else {
                var error = new call_1.ServiceErrorImpl(status.details);
                error.code = status.code;
                error.metadata = status.metadata;
                callback(error);
            }
        });
    };
    Client.prototype.checkOptionalUnaryResponseArguments = function (arg1, arg2, arg3) {
        if (arg1 instanceof Function) {
            return { metadata: new metadata_1.Metadata(), options: {}, callback: arg1 };
        }
        else if (arg2 instanceof Function) {
            if (arg1 instanceof metadata_1.Metadata) {
                return { metadata: arg1, options: {}, callback: arg2 };
            }
            else {
                return { metadata: new metadata_1.Metadata(), options: arg1, callback: arg2 };
            }
        }
        else {
            if (!((arg1 instanceof metadata_1.Metadata) && (arg2 instanceof Object) &&
                (arg3 instanceof Function))) {
                throw new Error('Incorrect arguments passed');
            }
            return { metadata: arg1, options: arg2, callback: arg3 };
        }
    };
    Client.prototype.makeUnaryRequest = function (method, serialize, deserialize, argument, metadata, options, callback) {
        (_a = this.checkOptionalUnaryResponseArguments(metadata, options, callback), metadata = _a.metadata, options = _a.options, callback = _a.callback);
        var call = this.channel.createStream(method, metadata, options);
        var emitter = new call_1.ClientUnaryCallImpl(call);
        var message = serialize(argument);
        var writeObj = { message: message };
        writeObj.flags = options.flags;
        call.write(writeObj);
        call.end();
        this.handleUnaryResponse(call, deserialize, callback);
        return emitter;
        var _a;
    };
    Client.prototype.makeClientStreamRequest = function (method, serialize, deserialize, metadata, options, callback) {
        (_a = this.checkOptionalUnaryResponseArguments(metadata, options, callback), metadata = _a.metadata, options = _a.options, callback = _a.callback);
        var call = this.channel.createStream(method, metadata, options);
        var stream = new call_1.ClientWritableStreamImpl(call, serialize);
        this.handleUnaryResponse(call, deserialize, callback);
        return stream;
        var _a;
    };
    Client.prototype.checkMetadataAndOptions = function (arg1, arg2) {
        var metadata;
        var options;
        if (arg1 instanceof metadata_1.Metadata) {
            metadata = arg1;
            if (arg2) {
                options = arg2;
            }
            else {
                options = {};
            }
        }
        else {
            if (arg1) {
                options = arg1;
            }
            else {
                options = {};
            }
            metadata = new metadata_1.Metadata();
        }
        return { metadata: metadata, options: options };
    };
    Client.prototype.makeServerStreamRequest = function (method, serialize, deserialize, argument, metadata, options) {
        (_a = this.checkMetadataAndOptions(metadata, options), metadata = _a.metadata, options = _a.options);
        var call = this.channel.createStream(method, metadata, options);
        var stream = new call_1.ClientReadableStreamImpl(call, deserialize);
        var message = serialize(argument);
        var writeObj = { message: message };
        writeObj.flags = options.flags;
        call.write(writeObj);
        call.end();
        return stream;
        var _a;
    };
    Client.prototype.makeBidiStreamRequest = function (method, serialize, deserialize, metadata, options) {
        (_a = this.checkMetadataAndOptions(metadata, options), metadata = _a.metadata, options = _a.options);
        var call = this.channel.createStream(method, metadata, options);
        var stream = new call_1.ClientDuplexStreamImpl(call, serialize, deserialize);
        return stream;
        var _a;
    };
    return Client;
}());
exports.Client = Client;

//# sourceMappingURL=client.js.map
